#include "sys/UserMemory.h"
#include "memory/Memory.h"
#include "std/string.h"
#include "utils/Addresses.h"

char* strdup_from_user(
    const char* user_string) // FIXME: This function is a little hacky. Use the obtain_user_ref and similar functions.
{
    uint64_t phys = VMM::get_physical((uint64_t)user_string);
    if (phys == (uint64_t)-1) { return nullptr; }
    return strdup((const char*)phys);
}

bool validate_user_readable_page(uintptr_t address)
{
    if (Memory::is_kernel_address(address)) return false;
    auto rc = VMM::get_flags(address);
    if (rc == (uint64_t)-1) return false;
    if (rc & MAP_USER) return true;
    return false;
}

bool validate_user_writable_page(uintptr_t address)
{
    if (Memory::is_kernel_address(address)) return false;
    auto rc = VMM::get_flags(address);
    if (rc == (uint64_t)-1) return false;
    if (rc & (MAP_USER | MAP_READ_WRITE)) return true;
    return false;
}

bool validate_user_read(uintptr_t address, size_t size)
{
    auto aligned = round_down_to_nearest_page(address);
    if (aligned != address) // Otherwise, we already do this check below.
    {
        if (!validate_user_readable_page(aligned)) return false;
    }
    while (size--)
    {
        if (address % PAGE_SIZE == 0)
        {
            if (!validate_user_readable_page(address)) return false;
        }
        address++;
    }
    return true;
}

bool validate_user_write(uintptr_t address, size_t size)
{
    auto aligned = round_down_to_nearest_page(address);
    if (aligned != address) // Otherwise, we already do this check below.
    {
        if (!validate_user_writable_page(aligned)) return false;
    }
    while (size--)
    {
        if (address % PAGE_SIZE == 0)
        {
            if (!validate_user_writable_page(address)) return false;
        }
        address++;
    }
    return true;
}